@@ -337,6 +337,7 @@
 #include <msg.h>
 #include <dict.h>
 #include <dict_cdb.h>
+#include <dict_debug.h>
 #include <dict_env.h>
 #include <dict_unix.h>
 #include <dict_tcp.h>
@@ -415,6 +416,7 @@ static const DICT_OPEN_INFO dict_open_info[] = {
     DICT_TYPE_LMDB, dict_lmdb_open, mkmap_lmdb_open,
 #endif
 #endif					/* !USE_DYNAMIC_MAPS */
+    DICT_TYPE_DEBUG, dict_debug_open, 0,
     0,
 };
 
@@ -610,7 +612,7 @@ void    dict_type_override(DICT *dict, const char *type)
   */
 int     main(int argc, char **argv)
 {
-    dict_cli(argc, argv);
+    dict_test(argc, argv);
     return (0);
 }
 
